from rich.text import Text

from textual.app import App, ComposeResult
from textual.widgets import DataTable

from matic.output_manager import OutputItem


class HistoryTable(DataTable):

    def __init__(self, cssid: str):
        super().__init__(id=cssid, zebra_stripes=True)
        self.history = []
        self.add_columns(*OutputItem.get_column_names())

    def add_cmd_item(self, output_item: OutputItem):
        self.add_row(*output_item.get_string_tuple(), label=output_item.get_idx_label())
        self.history.append(output_item)
        self.move_cursor(row=len(self.history))

    def get_cursor_cmd(self):
        if self.cursor_row < 0 or self.cursor_row >= len(self.history):
            return ""
        self.cmd_input_idx = self.cursor_row
        return self.history[self.cursor_row].cmd

    def get_prev_cmd(self) -> str:
        next_cursor_row = self.cursor_row - 1
        if next_cursor_row <= -1:
            next_cursor_row = len(self.history)
        self.move_cursor(row=next_cursor_row)
        return self.get_cursor_cmd()

    def get_next_cmd(self) -> str:
        next_cursor_row = self.cursor_row + 1
        if next_cursor_row >= len(self.history):
            next_cursor_row = 0
        self.move_cursor(row=next_cursor_row)
        return self.get_cursor_cmd()



from rich.syntax import Syntax
from rich.table import Table

from textual import events
from textual.app import App, ComposeResult
from textual.widgets import RichLog, Label, ListItem, ListView

from matic.output_manager import OutputItem


class History(ListView):

    def __init__(self):
        super().__init__()

    def add_cmd_item(self, output_item: OutputItem):
        output_str = output_item.to_colorful_string()
        self.append(ListItem(Label(output_str)))



from textual import on, work
from textual.app import App, ComposeResult
from textual.binding import Binding
from textual.containers import Vertical, Horizontal
from textual.widget import AwaitMount, Widget
from textual.widgets import Button, Footer, Input, Header
from textual import events
import asyncio

from .components.cmd_input import CmdInput
from .components.cmd_log import CmdLog
from .components.history import History
from .components.history_table import HistoryTable
from .output_manager import OutputManager, OutputItem

STATA_READY_SIGNAL = b'. \n'
MATA_READY_SIGNAL = b': \n'


class StatMatic(App, inherit_bindings=False):

    CSS_PATH = "./dom.tcss"

    BINDINGS = [
        Binding("ctrl+q", "quit", "Quit"),
    ]

    def __init__(self):
        super().__init__()
        self.cmd_input = CmdInput()
        self.cmd_log = CmdLog()
        self.footer = Footer()
        self.history = HistoryTable(cssid="history")
        self.process = None
        self.cmd_input.focus()
        self.cmd_input_default_border_title = "Ready for next command"
        self.cmd_input.border_title = self.cmd_input_default_border_title
        self.output_manager = OutputManager()

    async def on_mount(self) -> None:
        self.process = await asyncio.create_subprocess_exec(
            "/Applications/Stata/StataMP.app/Contents/MacOS/stata-mp",
            stdin=asyncio.subprocess.PIPE,
            stdout=asyncio.subprocess.PIPE)
        await self.action_submit()

    def compose(self) -> ComposeResult:
        with Horizontal():
            yield self.history
            with Vertical():
                yield self.cmd_log
                yield self.cmd_input
            yield self.footer

    async def quit(self) -> None:
        self.process.stdin.close()
        await self.process.wait()
        exit(0)

    async def read_output(self, output_item, timeout=0.2):
        """Read output from the process until a timeout occurs."""
        while True:
            try:
                line = await asyncio.wait_for(self.process.stdout.readline(), timeout=timeout)
                processed_line = line.decode().strip("\n")
                self.cmd_log.call_update(processed_line)
                output_item.append(processed_line)
            except asyncio.TimeoutError:
                self.process.stdin.write(b'\n')
                await self.process.stdin.drain()
                prompt_check = await self.process.stdout.readline()
                if prompt_check == STATA_READY_SIGNAL or prompt_check == MATA_READY_SIGNAL:
                    return
            if self.process.returncode is not None:
                exit(self.process.returncode)

    async def action_submit(self):
        # call an independent process
        # Send the string to the shell process
        self.cmd_input.border_title = ""
        self.process.stdin.write(self.cmd_input.value.encode() + b'\n')
        await self.process.stdin.drain()

        output_item = OutputItem(cmd=self.cmd_input.value)
        self.output_manager.append(output_item)

        self.cmd_input.clear()
        self.cmd_input.disabled = True

        # Get the output from the shell process
        await self.read_output(output_item)
        self.cmd_input.disabled = False
        self.cmd_input.focus()
        self.cmd_input.border_title = self.cmd_input_default_border_title

        if len(output_item.cmd) > 0:
            self.history.add_cmd_item(output_item)

    async def on_cmd_input_cmd_submitted(self) -> None:
        await self.action_submit()

    def on_key(self, event: events.Key) -> None:
        if self.focused.__class__ != HistoryTable:
            if event.key == "up":
                self.cmd_input.value = self.history.get_prev_cmd()
            elif event.key == "down":
                self.cmd_input.value = self.history.get_next_cmd()

    def on_data_table_cell_selected(self) -> None:
        self.cmd_input.value = str(self.history.get_cursor_cmd())




from rich.text import Text


class OutputItem:

    def __init__(self, cmd):
        self.lines = []
        self.cmd = cmd
        self.index = -1

    @staticmethod
    def get_column_names():
        return "Command", "Status"

    def append(self, line):
        self.lines.append(line)

    def get_string_tuple(self):
        return self.cmd, str("None")

    def get_idx_label(self):
        return str(self.index)

    def to_colorful_string(self):
        txt_1 = Text()
        txt_1.append(str(self.index), style="bold magenta")
        txt_1.append(":\t")
        txt_1.append(str(self.cmd), style="italic")
        return txt_1


class OutputManager:

    def __init__(self):
        self.arr = []
        self.cnt = 0

    def append(self, output_item: OutputItem):
        output_item.index = self.cnt
        self.cnt += 1
        self.arr.append(output_item)



from rich.console import Console
from rich.style import Style
from rich.syntax import Syntax
from rich.table import Table
from rich.text import Text
from rich.highlighter import RegexHighlighter

from textual import events
from textual.app import App, ComposeResult
from textual.widgets import RichLog


class CmdLog(RichLog):

    def __init__(self):
        super().__init__()

    def compose(self) -> ComposeResult:
        yield RichLog(highlight=True, markup=True)

    def call_update(self, slog: str):
        text_log = self.query_one(RichLog)
        text_log.write(slog)



import rich_click as click
from . import app


def build_cli() -> click.command:
    @click.command()
    def inner_cli() -> None:
        tui = app.StatMatic()
        tui.run()
        return
    fn = inner_cli
    return fn


def main():
    cli = build_cli()
    cli()



from textual import events
from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import VerticalScroll
from textual.document._document import EditResult
from textual.widgets import TextArea
from textual.widgets import Input
from textual.message import Message
from textual.widgets.text_area import Selection


class CmdInput(Input):

    DEFAULT_PLACEHOLDER = "Type <enter> to submit"

    class UseHistory(Message):

        def __init__(self, direction: str):
            self.direction = direction
            super().__init__()

    def __init__(self) -> None:
        super().__init__(value="", placeholder=self.DEFAULT_PLACEHOLDER)

    class CmdSubmitted(Message):

        def __init__(self) -> None:
            super().__init__()

    def clear(self) -> None:
        self.value = ""
        self.placeholder = self.DEFAULT_PLACEHOLDER

    def action_submit(self) -> None:
        self.post_message(self.CmdSubmitted())

    def input_cursor_up(self):
        self.post_message(self.UseHistory(direction="up"))

    def input_cursor_down(self):
        self.post_message(self.UseHistory(direction="down"))
